@@ -52,7 +52,7 @@ class QSocketNotifier;
 #include <sys/types.h> // time_t, ino_t
 #include <ctime>
 
-#define invalid_ctime ((time_t)-1)
+#define invalid_ctime (static_cast<time_t>(-1))
 
 #if HAVE_QFILESYSTEMWATCHER
 #include <QtCore/QFileSystemWatcher>
@@ -71,6 +71,21 @@ public:
     enum { NoChange = 0, Changed = 1, Created = 2, Deleted = 4 };
 
     struct Client {
+        Client(KDirWatch *inst, KDirWatch::WatchModes watchModes)
+            : instance(inst),
+            count(1),
+            watchingStopped(inst->isStopped()),
+            pending(NoChange),
+            m_watchModes(watchModes)
+        {}
+
+        // The compiler needs a copy ctor for Client when Entry is inserted into m_mapEntries
+        // (even though the vector of clients is empty at that point, so no performance penalty there)
+        //Client(const Client &) = delete;
+        //Client &operator=(const Client &) = delete;
+        //Client(Client &&) = default;
+        //Client &operator=(Client &&) = default;
+
         KDirWatch *instance;
         int count;
         // did the instance stop watching
@@ -83,22 +98,23 @@ public:
     class Entry
     {
     public:
-        // the last observed modification time
-        time_t m_ctime;
-        // the last observed link count
-        int m_nlink;
-        // last observed inode
-        ino_t m_ino;
-        entryStatus m_status;
-        entryMode m_mode;
-        bool isDir;
+        ~Entry();
         // instances interested in events
-        QList<Client *> m_clients;
+        std::vector<Client> m_clients;
         // nonexistent entries of this directory
         QList<Entry *> m_entries;
         QString path;
 
+        // the last observed modification time
+        time_t m_ctime;
+        // last observed inode
+        ino_t m_ino;
+        // the last observed link count
+        int m_nlink;
+        entryStatus m_status;
+        entryMode m_mode;
         int msecLeft, freq;
+        bool isDir;
 
         QString parentDirectory() const;
         void addClient(KDirWatch *, KDirWatch::WatchModes);
@@ -106,7 +122,7 @@ public:
         int clientCount() const;
         bool isValid()
         {
-            return m_clients.count() || m_entries.count();
+            return !m_clients.empty() || !m_entries.empty();
         }
 
         Entry *findSubEntry(const QString &path) const
@@ -116,14 +132,14 @@ public:
                     return sub_entry;
                 }
             }
-            return 0;
+            return nullptr;
         }
 
         bool dirty;
         void propagate_dirty();
 
-        QList<Client *> clientsForFileOrDir(const QString &tpath, bool *isDir) const;
-        QList<Client *> inotifyClientsForFileOrDir(bool isDir) const;
+        QList<const Client *> clientsForFileOrDir(const QString &tpath, bool *isDir) const;
+        QList<const Client *> inotifyClientsForFileOrDir(bool isDir) const;
 
 #if HAVE_FAM
         FAMRequest fr;
@@ -150,7 +166,7 @@ public:
     void useFreq(Entry *e, int newFreq);
     void addEntry(KDirWatch *instance, const QString &_path, Entry *sub_entry,
                   bool isDir, KDirWatch::WatchModes watchModes = KDirWatch::WatchDirOnly);
-    bool removeEntry(KDirWatch *instance, const QString &path, Entry *sub_entry);
+    void removeEntry(KDirWatch *instance, const QString &path, Entry *sub_entry);
     void removeEntry(KDirWatch *instance, Entry *e, Entry *sub_entry);
     bool stopEntryScan(KDirWatch *instance, Entry *e);
     bool restartEntryScan(KDirWatch *instance, Entry *e, bool notify);
@@ -164,17 +180,7 @@ public:
     void removeWatch(Entry *entry);
     Entry *entry(const QString &_path);
     int scanEntry(Entry *e);
-    void emitEvent(const Entry *e, int event, const QString &fileName = QString());
-
-    // Memory management - delete when last KDirWatch gets deleted
-    void ref()
-    {
-        m_ref++;
-    }
-    bool deref()
-    {
-        return (--m_ref == 0);
-    }
+    void emitEvent(Entry *e, int event, const QString &fileName = QString());
 
     static bool isNoisyFile(const char *filename);
 
@@ -193,7 +199,6 @@ public:
     int freq;
     int statEntries;
     int m_nfsPollInterval, m_PollInterval;
-    int m_ref;
     bool useStat(Entry *e);
 
     // removeList is allowed to contain any entry at most once
