@@ -4,6 +4,7 @@
  * Copyright (C) 2006 Nicolas GOUTTE <goutte@kde.org>
  * Copyright (C) 2008 Friedrich W. H. Kossebau <kossebau@kde.org>
  * Copyright (C) 2010 Teo Mrnjavac <teo@kde.org>
+ * Copyright (C) 2017 Harald Sitter <sitter@kde.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -47,7 +48,7 @@ Q_DECLARE_LOGGING_CATEGORY(KABOUTDATA)
 Q_LOGGING_CATEGORY(KABOUTDATA, "kf5.kcoreaddons.kaboutdata", QtWarningMsg)
 
 
-class KAboutPerson::Private
+class Q_DECL_HIDDEN KAboutPerson::Private
 {
 public:
     QString _name;
@@ -130,30 +131,30 @@ KAboutPerson KAboutPerson::fromJSON(const QJsonObject &obj)
 }
 
 
-class KAboutLicense::Private : public QSharedData
+class Q_DECL_HIDDEN KAboutLicense::Private : public QSharedData
 {
 public:
-    Private(LicenseKey licenseType, const KAboutData *aboutData);
-    Private(const KAboutData *aboutData);
+    Private(LicenseKey licenseType,
+            VersionRestriction versionRestriction,
+            const KAboutData *aboutData);
     Private(const Private &other);
-public:
-    LicenseKey       _licenseKey;
-    QString          _licenseText;
-    QString          _pathToLicenseTextFile;
+
+    QString spdxID() const;
+
+    LicenseKey _licenseKey;
+    QString _licenseText;
+    QString _pathToLicenseTextFile;
+    VersionRestriction _versionRestriction;
     // needed for access to the possibly changing copyrightStatement()
     const KAboutData *_aboutData;
 };
 
-KAboutLicense::Private::Private(LicenseKey licenseType, const KAboutData *aboutData)
+KAboutLicense::Private::Private(LicenseKey licenseType,
+                                VersionRestriction versionRestriction,
+                                const KAboutData *aboutData)
     : QSharedData(),
       _licenseKey(licenseType),
-      _aboutData(aboutData)
-{
-}
-
-KAboutLicense::Private::Private(const KAboutData *aboutData)
-    : QSharedData(),
-      _licenseKey(Unknown),
+      _versionRestriction(versionRestriction),
       _aboutData(aboutData)
 {
 }
@@ -163,16 +164,52 @@ KAboutLicense::Private::Private(const KAboutLicense::Private &other)
       _licenseKey(other._licenseKey),
       _licenseText(other._licenseText),
       _pathToLicenseTextFile(other._pathToLicenseTextFile),
+      _versionRestriction(other._versionRestriction),
       _aboutData(other._aboutData)
 {}
 
+QString KAboutLicense::Private::spdxID() const
+{
+    switch (_licenseKey) {
+    case KAboutLicense::GPL_V2:
+        return QStringLiteral("GPL-2.0");
+    case KAboutLicense::LGPL_V2:
+        return QStringLiteral("LGPL-2.0");
+    case KAboutLicense::BSDL:
+        return QStringLiteral("BSD-2-Clause");
+    case KAboutLicense::Artistic:
+        return QStringLiteral("Artistic-1.0");
+    case KAboutLicense::QPL_V1_0:
+        return QStringLiteral("QPL-1.0");
+    case KAboutLicense::GPL_V3:
+        return QStringLiteral("GPL-3.0");
+    case KAboutLicense::LGPL_V3:
+        return QStringLiteral("LGPL-3.0");
+    case KAboutLicense::LGPL_V2_1:
+        return QStringLiteral("LGPL-2.1");
+    case KAboutLicense::Custom:
+    case KAboutLicense::File:
+    case KAboutLicense::Unknown:
+        return QString();
+    }
+    return QString();
+}
+
+KAboutLicense::KAboutLicense(LicenseKey licenseType,
+                             VersionRestriction versionRestriction,
+                             const KAboutData *aboutData)
+    : d(new Private(licenseType, versionRestriction, aboutData))
+{
+
+}
+
 KAboutLicense::KAboutLicense(LicenseKey licenseType, const KAboutData *aboutData)
-    : d(new Private(licenseType, aboutData))
+    : d(new Private(licenseType, OnlyThisVersion, aboutData))
 {
 }
 
 KAboutLicense::KAboutLicense(const KAboutData *aboutData)
-    : d(new Private(aboutData))
+    : d(new Private(Unknown, OnlyThisVersion, aboutData))
 {
 }
 
@@ -280,6 +317,26 @@ QString KAboutLicense::text() const
     return result;
 }
 
+QString KAboutLicense::spdx() const
+{
+    // SPDX licenses are comprised of an identifier (e.g. GPL-2.0), an optional + to denote 'or
+    // later versions' and optional ' WITH $exception' to denote standardized exceptions from the
+    // core license. As we do not offer exceptions we effectively only return GPL-2.0 or GPL-2.0+,
+    // this may change in the future. To that end the documentation makes no assertations about the
+    // actual content of the SPDX license expression we return.
+    // Expressions can in theory also contain AND, OR and () to build constructs involving more than
+    // one license. As this is outside the scope of a single license object we'll ignore this here
+    // for now.
+    // The expecation is that the return value is only run through spec-compliant parsers, so this
+    // can potentially be changed.
+
+    auto id = d->spdxID();
+    if (id.isNull()) { // Guard against potential future changes which would allow 'Foo+' as input.
+        return id;
+    }
+    return d->_versionRestriction == OrLaterVersions ? id.append(QLatin1Char('+')) : id;
+}
+
 QString KAboutLicense::name(KAboutLicense::NameFormat formatName) const
 {
     QString licenseShort;
@@ -349,26 +406,25 @@ KAboutLicense KAboutLicense::byKeyword(const QString &rawKeyword)
 {
     // Setup keyword->enum dictionary on first call.
     // Use normalized keywords, by the algorithm below.
-    static QHash<QByteArray, KAboutLicense::LicenseKey> ldict;
-    if (ldict.isEmpty()) {
-        ldict.insert("gpl", KAboutLicense::GPL);
-        ldict.insert("gplv2", KAboutLicense::GPL_V2);
-        ldict.insert("gplv2+", KAboutLicense::GPL_V2);
-        ldict.insert("lgpl", KAboutLicense::LGPL);
-        ldict.insert("lgplv2", KAboutLicense::LGPL_V2);
-        ldict.insert("lgplv2+", KAboutLicense::LGPL_V2);
-        ldict.insert("bsd", KAboutLicense::BSDL);
-        ldict.insert("artistic", KAboutLicense::Artistic);
-        ldict.insert("qpl", KAboutLicense::QPL);
-        ldict.insert("qplv1", KAboutLicense::QPL_V1_0);
-        ldict.insert("qplv10", KAboutLicense::QPL_V1_0);
-        ldict.insert("gplv3", KAboutLicense::GPL_V3);
-        ldict.insert("gplv3+", KAboutLicense::GPL_V3);
-        ldict.insert("lgplv3", KAboutLicense::LGPL_V3);
-        ldict.insert("lgplv3+", KAboutLicense::LGPL_V3);
-        ldict.insert("lgplv21", KAboutLicense::LGPL_V2_1);
-        ldict.insert("lgplv21+", KAboutLicense::LGPL_V2_1);
-    }
+    static const QHash<QByteArray, KAboutLicense::LicenseKey> licenseDict {
+        { "gpl", KAboutLicense::GPL },
+        { "gplv2", KAboutLicense::GPL_V2 },
+        { "gplv2+", KAboutLicense::GPL_V2 },
+        { "lgpl", KAboutLicense::LGPL },
+        { "lgplv2", KAboutLicense::LGPL_V2 },
+        { "lgplv2+", KAboutLicense::LGPL_V2 },
+        { "bsd", KAboutLicense::BSDL },
+        { "artistic", KAboutLicense::Artistic },
+        { "qpl", KAboutLicense::QPL },
+        { "qplv1", KAboutLicense::QPL_V1_0 },
+        { "qplv10", KAboutLicense::QPL_V1_0 },
+        { "gplv3", KAboutLicense::GPL_V3 },
+        { "gplv3+", KAboutLicense::GPL_V3 },
+        { "lgplv3", KAboutLicense::LGPL_V3 },
+        { "lgplv3+", KAboutLicense::LGPL_V3 },
+        { "lgplv21", KAboutLicense::LGPL_V2_1 },
+        { "lgplv21+", KAboutLicense::LGPL_V2_1 }
+    };
 
     // Normalize keyword.
     QString keyword = rawKeyword;
@@ -376,12 +432,13 @@ KAboutLicense KAboutLicense::byKeyword(const QString &rawKeyword)
     keyword.remove(QLatin1Char(' '));
     keyword.remove(QLatin1Char('.'));
 
-    LicenseKey license = ldict.value(keyword.toLatin1(),
-                                     KAboutLicense::Custom);
-    return KAboutLicense(license, 0);
+    LicenseKey license = licenseDict.value(keyword.toLatin1(),
+                                           KAboutLicense::Custom);
+    auto restriction = keyword.endsWith(QLatin1Char('+')) ? OrLaterVersions : OnlyThisVersion;
+    return KAboutLicense(license, restriction, nullptr);
 }
 
-class KAboutData::Private
+class Q_DECL_HIDDEN KAboutData::Private
 {
 public:
     Private()
@@ -411,7 +468,7 @@ public:
     // can just give it a pointer, w/o any allocations.
     QByteArray _internalProgramName;
     QByteArray _version;
-    QByteArray _bugEmailAddress;
+    QByteArray _bugAddress;
 
     static QList<KAboutPerson> parseTranslators(const QString &translatorName, const QString &translatorEmail);
 
@@ -425,7 +482,7 @@ KAboutData::KAboutData(const QString &_componentName,
                        const QString &_copyrightStatement,
                        const QString &text,
                        const QString &homePageAddress,
-                       const QString &bugsEmailAddress
+                       const QString &bugAddress
                       )
     : d(new Private)
 {
@@ -445,7 +502,7 @@ KAboutData::KAboutData(const QString &_componentName,
     d->_copyrightStatement = _copyrightStatement;
     d->_otherText = text;
     d->_homepageAddress = homePageAddress;
-    d->_bugEmailAddress = bugsEmailAddress.toUtf8();
+    d->_bugAddress = bugAddress.toUtf8();
 
     QUrl homePageUrl(homePageAddress);
     if (!homePageUrl.isValid() || homePageUrl.scheme().isEmpty()) {
@@ -463,6 +520,9 @@ KAboutData::KAboutData(const QString &_componentName,
 
     d->organizationDomain = hostComponents.join(dotChar);
 
+    // KF6: do not set a default desktopFileName value here, but remove this code and leave it empty
+    // see KAboutData::desktopFileName() for detals
+
     // desktop file name is reverse domain name
     std::reverse(hostComponents.begin(), hostComponents.end());
     hostComponents.append(_componentName);
@@ -490,8 +550,10 @@ KAboutData::KAboutData(const QString &_componentName,
 
     // match behaviour of other constructors
     d->_licenseList.append(KAboutLicense(KAboutLicense::Unknown, this));
-    d->_bugEmailAddress = "submit@bugs.kde.org";
+    d->_bugAddress = "submit@bugs.kde.org";
     d->organizationDomain = QStringLiteral("kde.org");
+    // KF6: do not set a default desktopFileName value here, but remove this code and leave it empty
+    // see KAboutData::desktopFileName() for detals
     d->desktopFileName = QStringLiteral("org.kde.%1").arg(d->_componentName);
 }
 
@@ -641,18 +703,30 @@ KAboutData &KAboutData::setShortDescription(const QString &_shortDescription)
 
 KAboutData &KAboutData::setLicense(KAboutLicense::LicenseKey licenseKey)
 {
-    d->_licenseList[0] = KAboutLicense(licenseKey, this);
+    return setLicense(licenseKey, KAboutLicense::OnlyThisVersion);
+}
+
+KAboutData &KAboutData::setLicense(KAboutLicense::LicenseKey licenseKey,
+                                   KAboutLicense::VersionRestriction versionRestriction)
+{
+    d->_licenseList[0] = KAboutLicense(licenseKey, versionRestriction, this);
     return *this;
 }
 
 KAboutData &KAboutData::addLicense(KAboutLicense::LicenseKey licenseKey)
+{
+    return addLicense(licenseKey, KAboutLicense::OnlyThisVersion);
+}
+
+KAboutData &KAboutData::addLicense(KAboutLicense::LicenseKey licenseKey,
+                                   KAboutLicense::VersionRestriction versionRestriction)
 {
     // if the default license is unknown, overwrite instead of append
     KAboutLicense &firstLicense = d->_licenseList[0];
     if (d->_licenseList.count() == 1 && firstLicense.d->_licenseKey == KAboutLicense::Unknown) {
-        firstLicense = KAboutLicense(licenseKey, this);
+        firstLicense = KAboutLicense(licenseKey, versionRestriction, this);
     } else {
-        d->_licenseList.append(KAboutLicense(licenseKey, this));
+        d->_licenseList.append(KAboutLicense(licenseKey, versionRestriction, this));
     }
     return *this;
 }
@@ -677,7 +751,7 @@ KAboutData &KAboutData::setHomepage(const QString &homepage)
 
 KAboutData &KAboutData::setBugAddress(const QByteArray &_bugAddress)
 {
-    d->_bugEmailAddress = _bugAddress;
+    d->_bugAddress = _bugAddress;
     return *this;
 }
 
@@ -774,7 +848,7 @@ QString KAboutData::homepage() const
 
 QString KAboutData::bugAddress() const
 {
-    return QString::fromUtf8(d->_bugEmailAddress.constData());
+    return QString::fromUtf8(d->_bugAddress.constData());
 }
 
 QString KAboutData::organizationDomain() const
@@ -787,10 +861,10 @@ QString KAboutData::organizationDomain() const
 /// for the bug mail address. Used in particular for KCrash.
 const char *KAboutData::internalBugAddress() const
 {
-    if (d->_bugEmailAddress.isEmpty()) {
-        return 0;
+    if (d->_bugAddress.isEmpty()) {
+        return nullptr;
     }
-    return d->_bugEmailAddress.constData();
+    return d->_bugAddress.constData();
 }
 
 QList<KAboutPerson> KAboutData::authors() const
@@ -912,12 +986,29 @@ KAboutData &KAboutData::setDesktopFileName(const QString &desktopFileName)
 QString KAboutData::desktopFileName() const
 {
     return d->desktopFileName;
+    // KF6: switch to this code and adapt API dox
+#if 0
+    // if desktopFileName has been explicitely set, use that value
+    if (!d->desktopFileName.isEmpty()) {
+        return d->desktopFileName;
+    }
+
+    // return a string calculated on-the-fly from the current org domain & component name
+    const QChar dotChar(QLatin1Char('.'));
+    QStringList hostComponents = d->organizationDomain.split(dotChar);
+
+    // desktop file name is reverse domain name
+    std::reverse(hostComponents.begin(), hostComponents.end());
+    hostComponents.append(componentName());
+
+    return hostComponents.join(dotChar);
+#endif
 }
 
 class KAboutDataRegistry
 {
 public:
-    KAboutDataRegistry() : m_appData(0) {}
+    KAboutDataRegistry() : m_appData(nullptr) {}
     ~KAboutDataRegistry()
     {
         delete m_appData;
@@ -1035,7 +1126,7 @@ const KAboutData *KAboutData::applicationDataPointer()
     if (s_registry.exists()) {
         return s_registry->m_appData;
     }
-    return 0;
+    return nullptr;
 }
 
 bool KAboutData::setupCommandLine(QCommandLineParser *parser)
