@@ -32,9 +32,9 @@
 
 #include <limits.h>
 
-#include <QDebug>
+#include "kcoreaddons_debug.h"
 
-static KTextToHTMLEmoticonsInterface *s_emoticonsInterface = 0;
+static KTextToHTMLEmoticonsInterface *s_emoticonsInterface = nullptr;
 
 static void loadEmoticonsPlugin()
 {
@@ -69,7 +69,7 @@ KTextToHTMLHelper::KTextToHTMLHelper(const QString &plainText, int pos, int maxU
 {
 }
 
-KTextToHTMLEmoticonsInterface* KTextToHTMLHelper::emoticonsInterface()
+KTextToHTMLEmoticonsInterface* KTextToHTMLHelper::emoticonsInterface() const
 {
     if (!s_emoticonsInterface) {
         loadEmoticonsPlugin();
@@ -109,7 +109,7 @@ QString KTextToHTMLHelper::getEmailAddress()
         // determine the domain part of the email address
         int dotPos = INT_MAX;
         int end = mPos + 1;
-        while (end < (int)mText.length() &&
+        while (end < mText.length() &&
                 (mText[end].isLetterOrNumber() ||
                  mText[end] == QLatin1Char('@') || // allow @ to find invalid email addresses
                  mText[end] == QLatin1Char('.') ||
@@ -143,7 +143,7 @@ QString KTextToHTMLHelper::getEmailAddress()
     return address;
 }
 
-bool KTextToHTMLHelper::atUrl()
+bool KTextToHTMLHelper::atUrl() const
 {
     // the following characters are allowed in a dot-atom (RFC 2822):
     // a-z A-Z 0-9 . ! # $ % & ' * + - / = ? ^ _ ` { | } ~
@@ -156,25 +156,24 @@ bool KTextToHTMLHelper::atUrl()
              (allowedSpecialChars.indexOf(mText[mPos - 1]) != -1))) {
         return false;
     }
-
     QChar ch = mText[mPos];
     return
-        (ch == QLatin1Char('h') && (mText.mid(mPos, 7) == QLatin1String("http://") ||
-                                    mText.mid(mPos, 8) == QLatin1String("https://"))) ||
-        (ch == QLatin1Char('v') && mText.mid(mPos, 6) == QLatin1String("vnc://")) ||
-        (ch == QLatin1Char('f') && (mText.mid(mPos, 7) == QLatin1String("fish://") ||
-                                    mText.mid(mPos, 6) == QLatin1String("ftp://") ||
-                                    mText.mid(mPos, 7) == QLatin1String("ftps://"))) ||
-        (ch == QLatin1Char('s') && (mText.mid(mPos, 7) == QLatin1String("sftp://") ||
-                                    mText.mid(mPos, 6) == QLatin1String("smb://"))) ||
-        (ch == QLatin1Char('m') && mText.mid(mPos, 7) == QLatin1String("mailto:")) ||
-        (ch == QLatin1Char('w') && mText.mid(mPos, 4) == QLatin1String("www.")) ||
-        (ch == QLatin1Char('f') && (mText.mid(mPos, 4) == QLatin1String("ftp.") ||
-                                    mText.mid(mPos, 7) == QLatin1String("file://"))) ||
-        (ch == QLatin1Char('n') && mText.mid(mPos, 5) == QLatin1String("news:"));
+        (ch == QLatin1Char('h') && (mText.midRef(mPos, 7) == QLatin1String("http://") ||
+                                    mText.midRef(mPos, 8) == QLatin1String("https://"))) ||
+        (ch == QLatin1Char('v') && mText.midRef(mPos, 6) == QLatin1String("vnc://")) ||
+        (ch == QLatin1Char('f') && (mText.midRef(mPos, 7) == QLatin1String("fish://") ||
+                                    mText.midRef(mPos, 6) == QLatin1String("ftp://") ||
+                                    mText.midRef(mPos, 7) == QLatin1String("ftps://"))) ||
+        (ch == QLatin1Char('s') && (mText.midRef(mPos, 7) == QLatin1String("sftp://") ||
+                                    mText.midRef(mPos, 6) == QLatin1String("smb://"))) ||
+        (ch == QLatin1Char('m') && mText.midRef(mPos, 7) == QLatin1String("mailto:")) ||
+        (ch == QLatin1Char('w') && mText.midRef(mPos, 4) == QLatin1String("www.")) ||
+        (ch == QLatin1Char('f') && (mText.midRef(mPos, 4) == QLatin1String("ftp.") ||
+                                    mText.midRef(mPos, 7) == QLatin1String("file://"))) ||
+        (ch == QLatin1Char('n') && mText.midRef(mPos, 5) == QLatin1String("news:"));
 }
 
-bool KTextToHTMLHelper::isEmptyUrl(const QString &url)
+bool KTextToHTMLHelper::isEmptyUrl(const QString &url) const
 {
     return url.isEmpty() ||
            url == QLatin1String("http://") ||
@@ -192,7 +191,7 @@ bool KTextToHTMLHelper::isEmptyUrl(const QString &url)
            url == QLatin1String("news://");
 }
 
-QString KTextToHTMLHelper::getUrl()
+QString KTextToHTMLHelper::getUrl(bool *badurl)
 {
     QString url;
     if (atUrl()) {
@@ -228,11 +227,40 @@ QString KTextToHTMLHelper::getUrl()
 
         url.reserve(mMaxUrlLen);    // avoid allocs
         int start = mPos;
-        while ((mPos < (int)mText.length()) &&
+        bool previousCharIsSpace = false;
+        bool previousCharIsADoubleQuote = false;
+        bool previousIsAnAnchor = false;
+        while ((mPos < mText.length()) &&
                 (mText[mPos].isPrint() || mText[mPos].isSpace()) &&
                 ((afterUrl.isNull() && !mText[mPos].isSpace()) ||
                  (!afterUrl.isNull() && mText[mPos] != afterUrl))) {
-            if (!mText[mPos].isSpace()) {     // skip whitespace
+            if (mText[mPos].isSpace()) {
+                previousCharIsSpace = true;
+            } else if (!previousIsAnAnchor && mText[mPos] == QLatin1Char('[')) {
+                break;
+            } else if (!previousIsAnAnchor && mText[mPos] == QLatin1Char(']')) {
+                break;
+            } else { // skip whitespace
+                if (previousCharIsSpace && mText[mPos] == QLatin1Char('<')) {
+                    url.append(QLatin1Char(' '));
+                    break;
+                }
+                previousCharIsSpace = false;
+                if (mText[mPos] == QLatin1Char('>') && previousCharIsADoubleQuote) {
+                    //it's an invalid url
+                    if (badurl) {
+                        *badurl = true;
+                    }
+                    return QString();
+                }
+                if (mText[mPos] == QLatin1Char('"')) {
+                    previousCharIsADoubleQuote = true;
+                } else {
+                    previousCharIsADoubleQuote = false;
+                }
+                if (mText[mPos] == QLatin1Char('#')) {
+                    previousIsAnAnchor = true;
+                }
                 url.append(mText[mPos]);
                 if (url.length() > mMaxUrlLen) {
                     break;
@@ -255,8 +283,7 @@ QString KTextToHTMLHelper::getUrl()
     //       a dot to finish the sentence. That would lead the parser to include the dot in the url,
     //       even though that is not wanted. So work around that here.
     //       Most real-life URLs hopefully don't end with dots or commas.
-    QList<QChar> wordBoundaries;
-    wordBoundaries << QLatin1Char('.') << QLatin1Char(',') << QLatin1Char(':') << QLatin1Char('!') << QLatin1Char('?') << QLatin1Char(')') << QLatin1Char('>');
+    static const QString wordBoundaries = QStringLiteral(".,:!?)>");
     if (url.length() > 1) {
         do {
             if (wordBoundaries.contains(url.at(url.length() - 1))) {
@@ -267,7 +294,6 @@ QString KTextToHTMLHelper::getUrl()
             }
         } while (url.length() > 1);
     }
-
     return url;
 }
 
@@ -308,7 +334,7 @@ QString KTextToHTMLHelper::highlightedText()
 }
 
 
-QString KTextToHTMLHelper::pngToDataUrl(const QString &iconPath)
+QString KTextToHTMLHelper::pngToDataUrl(const QString &iconPath) const
 {
     if (iconPath.isEmpty()) {
         return QString();
@@ -321,7 +347,7 @@ QString KTextToHTMLHelper::pngToDataUrl(const QString &iconPath)
 
     QByteArray ba = pngFile.readAll();
     pngFile.close();
-    return QString(QStringLiteral("data:image/png;base64,%1")).arg(QLatin1String(ba.toBase64().constData()));
+    return QStringLiteral("data:image/png;base64,%1").arg(QLatin1String(ba.toBase64().constData()));
 }
 
 
@@ -330,12 +356,12 @@ QString KTextToHTML::convertToHtml(const QString &plainText, const KTextToHTML::
     KTextToHTMLHelper helper(plainText, maxUrlLen, maxAddressLen);
 
     QString str;
-    QString result((QChar *)0, (int)helper.mText.length() * 2);
+    QString result(static_cast<QChar *>(nullptr), helper.mText.length() * 2);
     QChar ch;
     int x;
     bool startOfLine = true;
 
-    for (helper.mPos = 0, x = 0; helper.mPos < (int)helper.mText.length();
+    for (helper.mPos = 0, x = 0; helper.mPos < helper.mText.length();
             ++helper.mPos, ++x) {
         ch = helper.mText[helper.mPos];
         if (flags & PreserveSpaces) {
@@ -401,7 +427,27 @@ QString KTextToHTML::convertToHtml(const QString &plainText, const KTextToHTML::
         } else {
             const int start = helper.mPos;
             if (!(flags & IgnoreUrls)) {
-                str = helper.getUrl();
+                bool badUrl = false;
+                str = helper.getUrl(&badUrl);
+                if (badUrl) {
+                    QString resultBadUrl;
+                    const int helperTextSize(helper.mText.count());
+                    for (int i = 0; i < helperTextSize; ++i) {
+                        const QChar chBadUrl = helper.mText[i];
+                        if (chBadUrl == QLatin1Char('&')) {
+                            resultBadUrl += QLatin1String("&amp;");
+                        } else if (chBadUrl == QLatin1Char('"')) {
+                            resultBadUrl += QLatin1String("&quot;");
+                        } else if (chBadUrl == QLatin1Char('<')) {
+                            resultBadUrl += QLatin1String("&lt;");
+                        } else if (chBadUrl == QLatin1Char('>')) {
+                            resultBadUrl += QLatin1String("&gt;");
+                        } else {
+                            resultBadUrl += chBadUrl;
+                        }
+                    }
+                    return resultBadUrl;
+                }
                 if (!str.isEmpty()) {
                     QString hyperlink;
                     if (str.left(4) == QLatin1String("www.")) {
@@ -446,12 +492,12 @@ QString KTextToHTML::convertToHtml(const QString &plainText, const KTextToHTML::
     }
 
     if (flags & ReplaceSmileys) {
-        QStringList exclude;
-        exclude << QStringLiteral("(c)") << QStringLiteral("(C)") << QStringLiteral("&gt;:-(") << QStringLiteral("&gt;:(") << QStringLiteral("(B)") << QStringLiteral("(b)") << QStringLiteral("(P)") << QStringLiteral("(p)");
-        exclude << QStringLiteral("(O)") << QStringLiteral("(o)") << QStringLiteral("(D)") << QStringLiteral("(d)") << QStringLiteral("(E)") << QStringLiteral("(e)") << QStringLiteral("(K)") << QStringLiteral("(k)");
-        exclude << QStringLiteral("(I)") << QStringLiteral("(i)") << QStringLiteral("(L)") << QStringLiteral("(l)") << QStringLiteral("(8)") << QStringLiteral("(T)") << QStringLiteral("(t)") << QStringLiteral("(G)");
-        exclude << QStringLiteral("(g)") << QStringLiteral("(F)") << QStringLiteral("(f)") << QStringLiteral("(H)");
-        exclude << QStringLiteral("8)") << QStringLiteral("(N)") << QStringLiteral("(n)") << QStringLiteral("(Y)") << QStringLiteral("(y)") << QStringLiteral("(U)") << QStringLiteral("(u)") << QStringLiteral("(W)") << QStringLiteral("(w)");
+        const QStringList exclude = { QStringLiteral("(c)"), QStringLiteral("(C)"), QStringLiteral("&gt;:-("), QStringLiteral("&gt;:("), QStringLiteral("(B)"), QStringLiteral("(b)"), QStringLiteral("(P)"), QStringLiteral("(p)")
+                                      , QStringLiteral("(O)"), QStringLiteral("(o)"), QStringLiteral("(D)"), QStringLiteral("(d)"), QStringLiteral("(E)"), QStringLiteral("(e)"), QStringLiteral("(K)"), QStringLiteral("(k)")
+                                      , QStringLiteral("(I)"), QStringLiteral("(i)"), QStringLiteral("(L)"), QStringLiteral("(l)"), QStringLiteral("(8)"), QStringLiteral("(T)"), QStringLiteral("(t)"), QStringLiteral("(G)")
+                                      , QStringLiteral("(g)"), QStringLiteral("(F)"), QStringLiteral("(f)"), QStringLiteral("(H)")
+                                      , QStringLiteral("8)"), QStringLiteral("(N)"), QStringLiteral("(n)"), QStringLiteral("(Y)"), QStringLiteral("(y)"), QStringLiteral("(U)"), QStringLiteral("(u)"), QStringLiteral("(W)"), QStringLiteral("(w)")
+                                      , QStringLiteral("(6)")};
 
         result = helper.emoticonsInterface()->parseEmoticons(result, true, exclude);
     }
